/*
 * dbsSTAT
 * db_STRESS STAT collecting @dim_STAT Project
 */

#include <stdio.h>
#include "anySTAT.h"

#define HEADER \
  puts("\n_______Group-ID__    Tot  Rds  Wrs    SEL1    SEL2     DEL     INS     UPD C/N" \
  );


char *tabSRC= NULL;
char *tabTMP= NULL;
SessionSTAT *tab1, *tab2;


SHM_init( char *skey, int MaxID )
{
 char *kstr;
 int key, shmid;

 if( (kstr= skey) == NULL )
   if( (kstr= (char *)getenv( "SHMKEY" )) == NULL)  kstr= DEF_SHMKEY;
 key= atoi( kstr );

 if( (shmid= shmget( key, SZ * MaxID, IPC_CREAT | 0777 )) == -1 )
 {
  puts( "### Cannot get SHM..." );
  exit( -2 );
 }

 if( (tabSRC= (char *)shmat( shmid, 0, 0 )) == (char *)-1 )
 {
  puts( "### Cannot attach SHM..." );
  exit( -2 );
 }

 tabTMP= (char *)calloc( MaxID, SZ );
 tab1= (SessionSTAT *)calloc( MaxID, sizeof( SessionSTAT ) );
 tab2= (SessionSTAT *)calloc( MaxID, sizeof( SessionSTAT ) );
}


Usage( char *prog )
{
 printf( "\n%s (dim) v.1.3\n\nUsage: %s key MaxID timeout [skip_every_N]\n", prog, prog );
 exit(-1);
}


GetSTAT( char *tab, SessionSTAT *stat, int max )
{
 char group[40];
 int i, check;

 for( i= 0; i < max; i++, tab += SZ, stat++ )
 {
  if( strlen( tab ) < 10 ) continue;
  sscanf( tab, "%s %d %d %d %d %d %d %d %d", 
     stat->group, &(stat->qr), &(stat->qw), 
     &(stat->s1), &(stat->s2), &(stat->del), &(stat->ins), &(stat->upd), &check );
  if( check != stat->qr )  stat->colls++;
 }
}


main( int argc, char *argv[] )
{
 long timeout, max, tm, tm2, Skip= 0;
 int i, n, r, w, s1, s2, ins, del, upd, nnR, nnW;
 SessionSTAT *t1, *t2, all;

 if( argc < 4 )  Usage( argv[0] );
 
 timeout= atoi( argv[3] );
 max= atoi( argv[2] );
 SHM_init( argv[1], max );
 if( argc > 4 )  Skip= atoi( argv[4] );

 time( &tm );
 tm += timeout;

 memcpy( tabTMP, tabSRC, SZ*max );
 GetSTAT( tabTMP, tab1, max );

 while( 1 )
 {
  time( &tm2 );
  sleep( tm - tm2 );
  tm += timeout;

  memset( &all, 0, sizeof( SessionSTAT ) ); 
  nnR= nnW= 0;
  memcpy( tabTMP, tabSRC, SZ*max );
  GetSTAT( tabTMP, tab2, max );
  HEADER;

  for( t1= tab1, t2= tab2, i= 0; i < max; i++, t1++, t2++ )
  {
   if( t2->qr == t1->qr )  continue;  //not active
   if( t2->qr < t1->qr || t2->qw < t1->qw )  memcpy( t1, t2, sizeof( SessionSTAT ) ); 

   r= t2->qr - t1->qr;
   w= t2->qw - t1->qw;
   if( r < 0 || w < 0 )  r= w= 0;
   s1= r == 0 ? 0 : (t2->s1 - t1->s1) / r;   
   s2= r == 0 ? 0 : (t2->s2 - t1->s2) / r;   
   del= w == 0 ? 0 : (t2->del - t1->del)/ w; 
   ins= w == 0 ? 0 : (t2->ins - t1->ins) / w;
   upd= w == 0 ? 0 : (t2->upd - t1->upd) / w;
   if( s1 < 0 || s2 < 0 )  s1= s2= 0;
   if( ins < 0 || del < 0 || upd < 0 )  del= ins= upd= 0;

   if( Skip == 0 || Skip % (i+1) == 0 ) // print details...
    printf( "%12s-%04d %6d %4d %4d %7d %7d %7d %7d %7d %3d\n",
    t2->group, i+1, 
    (r + w), r, w, s1, s2, del, ins, upd,
    (t2->colls - t1->colls)
   );

   if( r > 0 ) 
   {
     all.qr += r;
     all.s1 += s1;
     all.s2 += s2;
     nnR++;
   }

   if( w > 0 ) 
   {
    all.qw += w;
    all.del += del;
    all.ins += ins;
    all.upd += upd;
    nnW++;
   }

   all.colls++;
   memcpy( t1, t2, sizeof( SessionSTAT ) ); 
  }
  
  printf( "%12s-sum  %6d %4d %4d %7d %7d %7d %7d %7d %3d\n",
     "ALL", all.qr + all.qw, all.qr, all.qw, 
     nnR == 0 ? 0 : all.s1 / nnR,
     nnR == 0 ? 0 : all.s2 / nnR,
     nnW == 0 ? 0 : all.del / nnW,
     nnW == 0 ? 0 : all.ins / nnW,
     nnW == 0 ? 0 : all.upd / nnW,
     all.colls
  );

  printf( "%12s-tps  %6d %4d %4d %7d %7d %7d %7d %7d %3d\n",
     "ALL", (all.qr + all.qw) / timeout, all.qr / timeout, all.qw / timeout, 
     nnR == 0 ? 0 : all.s1 / nnR,
     nnR == 0 ? 0 : all.s2 / nnR,
     nnW == 0 ? 0 : all.del / nnW,
     nnW == 0 ? 0 : all.ins / nnW,
     nnW == 0 ? 0 : all.upd / nnW,
     all.colls
  );
  
  fflush( stdout );
 }
}

